@@ -240,6 +240,18 @@ setMethod("corpusQuery", "KorAPConnection",
     }
   })
 
+#' @importFrom purrr map
+repair_data_strcuture <- function(x) {
+  if (is.list(x))
+    as.character (purrr::map(x, ~ if (length(.x) > 1) {
+      paste(.x, collapse = " ")
+    } else {
+      .x
+    }))
+  else
+    ifelse(is.na(x), "", x)
+}
+
 #' Fetch the next bunch of results of a KorAP query.
 #'
 #' **`fetchNext`** fetches the next bunch of results of a KorAP query.
@@ -263,28 +275,36 @@ setMethod("corpusQuery", "KorAPConnection",
 #'
 #' @aliases fetchNext
 #' @rdname KorAPQuery-class
-#' @importFrom dplyr rowwise bind_rows select summarise n
+#' @importFrom dplyr rowwise mutate bind_rows select summarise n select
+#' @importFrom tibble enframe add_column
+#' @importFrom stringr word
+#' @importFrom tidyr unnest unchop pivot_wider
+#' @importFrom purrr map
 #' @export
 setMethod("fetchNext", "KorAPQuery", function(kqo,
                                               offset = kqo@nextStartIndex,
                                               maxFetch = maxResultsPerPage,
                                               verbose = kqo@korapConnection@verbose,
                                               randomizePageOrder = FALSE) {
+  # https://stackoverflow.com/questions/8096313/no-visible-binding-for-global-variable-note-in-r-cmd-check
+  results <- key <- name <- pubDate <- tmp_positions <- 0
+
   if (kqo@totalResults == 0 || offset >= kqo@totalResults) {
     return(kqo)
   }
-
+  use_korap_api <- Sys.getenv("USE_KORAP_API", unset = NA)
   page <- kqo@nextStartIndex / maxResultsPerPage + 1
-  results <- 0
-  pubDate <- NULL # https://stackoverflow.com/questions/8096313/no-visible-binding-for-global-variable-note-in-r-cmd-check
   collectedMatches <- kqo@collectedMatches
 
   if (randomizePageOrder) {
     pages <- head(sample.int(ceiling(kqo@totalResults / maxResultsPerPage)), maxFetch) - 1
   }
 
+  if(is.null(collectedMatches)) {
+    collectedMatches <- data.frame()
+  }
   repeat {
-    page = length(collectedMatches[,1]) %/% maxResultsPerPage + 1
+    page = nrow(collectedMatches) %/% maxResultsPerPage + 1
     currentOffset = ifelse(randomizePageOrder, pages[page],  page - 1) * maxResultsPerPage
     query <- paste0(kqo@requestUrl, '&count=', min(if (!is.na(maxFetch)) maxFetch - results else maxResultsPerPage, maxResultsPerPage) ,'&offset=', currentOffset, '&cutoff=true')
     res <- apiCall(kqo@korapConnection, query)
@@ -292,14 +312,38 @@ setMethod("fetchNext", "KorAPQuery", function(kqo,
       break
     }
 
+    if ("fields" %in% colnames(res$matches) && (is.na(use_korap_api) || as.numeric(use_korap_api) >= 1.0)) {
+      if (verbose) cat("Using fields API: ")
+      currentMatches <- res$matches$fields %>%
+        purrr::map(~ mutate(.x, value = repair_data_strcuture(value))) %>%
+        tibble::enframe() %>%
+        tidyr::unnest(cols = value) %>%
+        tidyr::pivot_wider(names_from = key, id_cols = name, names_repair = "unique") %>%
+        dplyr::select(-name)
+      if("snippet" %in% colnames(res$matches)) {
+        currentMatches$snippet <- res$matches$snippet
+      }
+      if ("tokens" %in% colnames(res$matches)) {
+        currentMatches$tokens <- res$matches$tokens
+      }
+    } else {
+      currentMatches <- res$matches
+    }
+
     for (field in kqo@fields) {
-      if (!field %in% colnames(res$matches)) {
-        res$matches[, field] <- NA
+      if (!field %in% colnames(currentMatches)) {
+        currentMatches[, field] <- NA
       }
     }
-    currentMatches <-
-      res$matches %>%
-      dplyr::select(kqo@fields)
+    currentMatches <- currentMatches %>%
+      select(kqo@fields) %>%
+      mutate(
+        tmp_positions = gsub(".*-p(\\d+)-(\\d+).*", "\\1 \\2", res$matches$matchID),
+        matchStart = as.integer(stringr::word(tmp_positions, 1)),
+        matchEnd = as.integer(stringr::word(tmp_positions, 2)) - 1
+      ) %>%
+      select(-tmp_positions)
+
     if (!is.list(collectedMatches)) {
       collectedMatches <- currentMatches
     } else {
@@ -308,7 +352,7 @@ setMethod("fetchNext", "KorAPQuery", function(kqo,
     if (verbose) {
       cat(paste0(
         "Retrieved page ",
-        ceiling(length(collectedMatches[, 1]) / res$meta$itemsPerPage),
+        ceiling(nrow(collectedMatches) / res$meta$itemsPerPage),
         "/",
         if (!is.na(maxFetch) && maxFetch < kqo@totalResults)
           sprintf("%d (%d)", ceiling(maxFetch / res$meta$itemsPerPage), ceiling(kqo@totalResults / res$meta$itemsPerPage))
@@ -321,7 +365,7 @@ setMethod("fetchNext", "KorAPQuery", function(kqo,
     }
     page <- page + 1
     results <- results + res$meta$itemsPerPage
-    if (length(collectedMatches[,1]) >= kqo@totalResults || (!is.na(maxFetch) && results >= maxFetch)) {
+    if (nrow(collectedMatches) >= kqo@totalResults || (!is.na(maxFetch) && results >= maxFetch)) {
       break
     }
   }
@@ -407,11 +451,37 @@ setMethod("frequencyQuery", "KorAPConnection",
       ci(conf.level = conf.level)
 })
 
+#' buildWebUIRequestUrlFromString
+#'
+#' @rdname KorAPQuery-class
+#' @importFrom urltools url_encode
+#' @export
+buildWebUIRequestUrlFromString <- function(KorAPUrl,
+                                 query,
+                                 vc = "",
+                                 ql = "poliqarp"
+) {
+  if ("KorAPConnection" %in% class(KorAPUrl)) {
+    KorAPUrl <- KorAPUrl@KorAPUrl
+  }
+
+  request <-
+    paste0(
+      '?q=',
+      urltools::url_encode(enc2utf8(as.character(query))),
+      ifelse(vc != '',
+             paste0('&cq=', urltools::url_encode(enc2utf8(vc))),
+             ''),
+      '&ql=',
+      ql
+    )
+  paste0(KorAPUrl, request)
+}
 
 #' buildWebUIRequestUrl
 #'
 #' @rdname KorAPQuery-class
-#' @importFrom urltools url_encode
+#' @importFrom httr parse_url
 #' @export
 buildWebUIRequestUrl <- function(kco,
                                  query = if (missing(KorAPUrl))
@@ -420,42 +490,9 @@ buildWebUIRequestUrl <- function(kco,
                                    httr::parse_url(KorAPUrl)$query$q,
                                  vc = if (missing(KorAPUrl)) "" else httr::parse_url(KorAPUrl)$query$cq,
                                  KorAPUrl,
-                                 metadataOnly = TRUE,
-                                 ql = if (missing(KorAPUrl)) "poliqarp" else httr::parse_url(KorAPUrl)$query$ql,
-                                 fields = c(
-                                   "corpusSigle",
-                                   "textSigle",
-                                   "pubDate",
-                                   "pubPlace",
-                                   "availability",
-                                   "textClass",
-                                   "snippet",
-                                   "tokens"
-                                 ),
-                                 accessRewriteFatal = TRUE) {
-  request <-
-    paste0(
-      '?q=',
-      urltools::url_encode(enc2utf8(as.character(query))),
-      ifelse(vc != '',
-        paste0('&cq=', urltools::url_encode(enc2utf8(vc))),
-        ''),
-      '&ql=',
-      ql
-    )
-  webUIRequestUrl <- paste0(kco@KorAPUrl, request)
-  requestUrl <- paste0(
-    kco@apiUrl,
-    'search',
-    request,
-    '&fields=',
-    paste(fields, collapse = ","),
-    if (metadataOnly)
-      '&access-rewrite-disabled=true'
-    else
-      ''
-  )
-  webUIRequestUrl
+                                 ql = if (missing(KorAPUrl)) "poliqarp" else httr::parse_url(KorAPUrl)$query$ql) {
+
+  buildWebUIRequestUrlFromString(kco@KorAPUrl, query, vc, ql)
 }
 
 #´ format()
